// mimulus.js - Immersive Hypertext Editing
// Author: Sean B. Palmer, inamidst.com
// Tested on Firefox 1.5 under OS X, only

function myKeypress(e) { 
   mySelection = window.getSelection(); 
   try { myRange = mySelection.getRangeAt(0); }
   catch(e) { return false; }

   if (e.metaKey) { 
      // The command key on OS X
      return false; 
   } else if (e.ctrlKey) { 
      switch(String.fromCharCode(e.charCode)) {
         // a all, f find, x cut, c copy, v paste 
         // e em, r strong, l a (m/n/return ends)
         // p pre, 1 h1, 2 h2, 3 h3, u ul
         // t toggle p/pre, i caret, s save
         // y remove formatting

         case 'i': // for "insert"
            myToggleCaret(); 
            e.preventDefault(); 
            e.stopPropagation(); 
            break; 

         case 's': // for "save"
            mySave(true); // true = prompt the save
            e.preventDefault();
            e.stopPropagation();
            break; 

         case 'e': // for "emphasize"
            myEmphasis(); 
            e.preventDefault();
            e.stopPropagation();
            break; 

         case 'r': // for "really emphasize"
            myStrong(); 
            e.preventDefault();
            e.stopPropagation();
            break; 

         case 'l': // for "link"
            myAnchor(); 
            e.preventDefault();
            e.stopPropagation();
            break; 

         case 'y': // for "yank inline"
            myRemoveInline(); 
            e.preventDefault();
            e.stopPropagation();
            break; 

         case 'n': // for "new paragraph"
            myParagraph(); 
            e.preventDefault();
            e.stopPropagation();
            break; 

         case 'u': // for "unordered list"
            myUnorderedList(); 
            e.preventDefault();
            e.stopPropagation();
            break; 

         case 'p': // for "preformatted"
            myPreformatted(); 
            e.preventDefault();
            e.stopPropagation();
            break; 

         case '1': // for "1st level heading
            myHeading(1); 
            e.preventDefault();
            e.stopPropagation();
            break; 

         case '2': // for "2nd level heading"
            myHeading(2); 
            e.preventDefault();
            e.stopPropagation();
            break; 

         case '3': // for "3rd level heading"
            myHeading(3); 
            e.preventDefault();
            e.stopPropagation();
            break; 

         case 't': // for "toggle"
            myTogglePreformatting(); 
            e.preventDefault();
            e.stopPropagation();
            break; 

         // @@ t: save and toggle caret browsing?
         // @@ n: no inline formatting
      }
   } else { 
      switch (e.keyCode) { 
         case e.DOM_VK_RETURN: 
            myReturn(); 
            break; 

         case e.DOM_VK_BACK_SPACE: 
            try { myMoveLeft(); }
            catch(exception) { 
               myPossiblyRemoveParagraph(); 
               e.preventDefault(); 
               e.stopPropagation(); 
               break; 
            }
            myDelete(); 
            myPossiblyRemoveParagraph(); 

            e.preventDefault();
            e.stopPropagation();
            break; 

         case e.DOM_VK_DELETE: 
            myDelete(); 

            e.preventDefault();
            e.stopPropagation();
            break; 

         default: 
            if (e.which != 0) { 
               myInsertChar(String.fromCharCode(e.charCode)); 
               e.preventDefault(); 
               e.stopPropagation(); 
               mySelection.collapse(mySelection.focusNode, 
                                    mySelection.focusOffset);
            }
      }
   }
}

function myKeyup() { 
   myUpdateStatus(); 
}

function myUpdateStatus() { 
   mySelection = window.getSelection();
   try { myRange = mySelection.getRangeAt(0); }
   catch(e) { 
      if (!myCaretMode) { 
         msg = 'To turn it on, select the window and press Ctrl+i.'; 
         window.status = 'Caret mode off. ' + msg; 
      } else { window.status = 'Caret mode on. Nothing selected.'; }
      return false; 
   }

   try { 
      var parent = myGetParent(myRange.startContainer); 
      var name = parent.tagName.toLowerCase(); 
      if (myCaretMode) { 
         if (myAutosaved) { // @@ ew, I know
            name = name + '. Autosaved!'; 
            myAutosaved = false; 
         }
         window.status = 'Caret mode on. Editing: ' + name; 
      } else { window.status = 'Caret mode off. Turn it on with Ctrl+i.'; }
   } catch(e) {} // alert(e); }
}

function myGetCaret() { 
   netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect');
   try {
      const PrefsCID = '@mozilla.org/preferences;1'
      var pref = new Components.Constructor(PrefsCID, 'nsIPref')();
      return pref.GetBoolPref('accessibility.browsewithcaret'); 
   } catch(e) { return false; }
}

function mySetCaret(boolean) { 
   // A hack for when F7 doesn't actually work...
   // Cf. http://www.mail-archive.com/dev-tech-xpcom@lists.mozilla.org/msg00531
   netscape.security.PrivilegeManager.enablePrivilege('UniversalXPConnect'); 
   try { 
      // Cf. http://xul.andreashalter.ch/
      const PrefsCID = '@mozilla.org/preferences;1'
      var pref = new Components.Constructor(PrefsCID, 'nsIPref')(); 
      pref.SetBoolPref('accessibility.browsewithcaret', boolean); 
      myCaretMode = boolean; 
   } catch(e) { alert(e); }
   myUpdateStatus(); 
}

function myToggleCaret() { 
   if (myCaretMode) { 
      mySetCaret(false); 
   } else { mySetCaret(true); }
}

function myInsertChar(c) { 
   // @@ Check that inserted chars are properly escaped (< and &)
   var parent = myGetParent(myRange.startContainer); 

   // The &nbsp; hack for empty paragraphs!
   if (parent.innerHTML == '&nbsp;') { 
      parent.innerHTML = c; 
      // if (myOldStartOffset == myRange.startOffset) {
      mySelectNode(parent.childNodes.item(0)); 
      myRange = mySelection.getRangeAt(0); 
      myRange.setEnd(myRange.endContainer, myRange.endOffset + 1);
      myRange.collapse(false);
      // }
      return; 
   }

   // Yet another hack for trailing spaces
   if (myRange.startOffset == myRange.startContainer.length) { 
      // If we're at the end of the current container, then...
      var html = parent.innerHTML; 

      if (html.substr(html.length - 6) == '&nbsp;' && html.length > 6) { 
         parent.innerHTML = html.substr(0, html.length - 6) + ' '; 
         mySelection.collapse(parent.lastChild, parent.lastChild.length); 
         myRange = mySelection.getRangeAt(0); 
      }

      if (c == ' ') { 
         parent.innerHTML = html + '&nbsp;'; 
         mySelection.collapse(parent.lastChild, parent.lastChild.length);
         return true; 
      }
   }

   try { myRange.deleteContents(); }
   catch(e) { return; }

   var myOldStartOffset = myRange.startOffset;
   if (myRange.startContainer.nodeType == myRange.startContainer.TEXT_NODE) {
      myRange.startContainer.insertData(myRange.startOffset, c);

      // The following basically advances the caret
      // @@ When selecting across elements, this selects everything
      if (myOldStartOffset == myRange.startOffset) { 
         myRange.setEnd(myRange.endContainer, myRange.endOffset + 1);
         myRange.collapse(false);
      }
   }

   mySelection.addRange(myRange); 
   // mySelection.collapse(mySelection.focusNode, mySelection.focusOffset);
   return c;
}

function mySelectNode(node) { 
   // Cf. http://www.opendarwin.org/pipermail/webkit-changes/2006-January.txt
   if (mySelection.selectAllChildren) { 
      mySelection.selectAllChildren(node); 
   }
}

function myGetMainParent(node) { 
   while (true) { 
      if (node.nodeType == node.TEXT_NODE) { 
         node = node.parentNode; 
         continue; 
      }

      var name = node.tagName.toLowerCase(); 
      if (name == 'p' || name == 'pre' || name == 'ul' || 
          name == 'h1' || name == 'h2' || name == 'h3') { 
         break
      }
      else { node = node.parentNode; }
   }

   return node; 
}

function myGetParent(node) { 
   var parent = node; // .parentNode; 

   while (true) { 
      if (parent.nodeType == parent.TEXT_NODE) { 
         parent = parent.parentNode; 
         continue; 
      }

      var name = parent.tagName.toLowerCase(); 
      if (name == 'p' || name == 'li' || name == 'pre' || 
          name == 'h1' || name == 'h2' || name == 'h3' ||  
          name == 'em' || name == 'strong' || name == 'a') { 
         break; 
      } else { var parent = parent.parentNode; }
   }

   return parent; 
}

function myFindParent(tagnames) { 
   var node = myRange.startContainer; 
   var matches = Object(); 
   for (var i in tagnames) { 
      matches[tagnames[i]] = 1; 
   }

   while (true) { 
      if (node.nodeType == node.TEXT_NODE) {
         var node = node.parentNode;
         continue;
      }
      else if (node == document.documentElement) { 
         return false; 
      }

      var name = node.tagName.toLowerCase();
      if (name in matches) { break }
      else { var node = node.parentNode; }
   }

   return node; 
}

function myGetParentBlock(node) {
   var parent = node.parentNode;

   while (true) {
      var name = parent.tagName.toLowerCase();
      if (name == 'p' || name == 'pre') { break; }
      else { var parent = parent.parentNode; }
   }

   return parent;
}

function myReturn() { 
   if (myRange.toString().length > 0) { 
      myDelete(); 
   } 
   else { 
      var parent = myGetParent(myRange.startContainer); 
      var name = parent.tagName.toLowerCase(); 

      if (name == 'pre') { 
         // When in <pre>, we need a way to be able to edit on multiple lines
         myInsertChar('\n'); 
      } else if (myRange.startOffset == myRange.startContainer.length) { 
         if (name == 'p' || name == 'h1' || name == 'h2' || name == 'h3') { 
            myNewParagraph(); 
         } else if (name == 'li') { 
            myNewListItem(); 
         } else if (name == 'em' || name == 'strong' || name == 'a') { 
            myExitElement(parent); 
         }
      }
   }
}

function myNewParagraph() { 
   var p = document.createElement('p'); 
   p.innerHTML = '&nbsp;'; 

   var parent = myGetMainParent(myRange.startContainer); 
   var grandparent = parent.parentNode; 

   if (parent.nextSibling) { 
      grandparent.insertBefore(p, parent.nextSibling); 
   } else { grandparent.appendChild(p); }

   mySelection.collapse(p.firstChild, 0); 
   mySave(false); // don't prompt, autosave
}

function myParagraph() { 
   var parent = myGetParent(myRange.startContainer); 
   var name = parent.tagName.toLowerCase(); 
   if (name == 'pre') {
      myNewParagraph();
   }
   else if (myRange.startOffset == myRange.startContainer.length) {
      if (name == 'p' || name == 'li' || 
          name == 'h1' || name == 'h2' || name == 'h3') { 
         myNewParagraph(); 
         // @@ Doesn't work in an &nbsp; paragraph
         // Possibly for the best, but...
      }
   }
}

function myNewListItem() { 
   var li = document.createElement('li'); 
   li.innerHTML = '&nbsp;'; 

   var parent = myGetParent(myRange.startContainer); 
   if (!(parent.tagName && parent.tagName.toLowerCase() == 'li')) { 
      return false; 
   }
   var grandparent = parent.parentNode; 

   if (parent.nextSibling) { 
      grandparent.insertBefore(li, parent.nextSibling); 
   } else { grandparent.appendChild(li); }

   mySelection.collapse(li.firstChild, 0); 
   mySave(false); // don't prompt, autosave
}

function myNewUnorderedList() { 
   var ul = document.createElement('ul'); 
   var li = document.createElement('li'); 
   li.innerHTML = '&nbsp;'; 
   ul.appendChild(li); 

   var parent = myGetParent(myRange.startContainer); 
   var grandparent = parent.parentNode; 

   if (parent.nextSibling) { 
      grandparent.insertBefore(ul, parent.nextSibling); 
   } else { grandparent.appendChild(ul); }

   mySelection.collapse(li.firstChild, 0); 
   mySave(false); // don't prompt, autosave
}

function myUnorderedList() { 
   var parent = myGetParent(myRange.startContainer); // pre, li, p, h1, h2, h3
   var name = parent.tagName.toLowerCase(); 
   if (name == 'pre') {
      myNewUnorderedList(); 
   // } else if (name == 'li') { 
   //    myNewInlineUnorderedList(); 
   // @@! Should sequential ULs be allowed? How about inline ULs?
   } else if (myRange.startOffset == myRange.startContainer.length) { 
      if (name == 'p' || name == 'h1' || name == 'h2' || name == 'h3') { 
         myNewUnorderedList(); 
      }
   }
}

function myNewPreformatted() { 
   var pre = document.createElement('pre'); 
   pre.innerHTML = '\n'; 

   var parent = myGetMainParent(myRange.startContainer); 
   var grandparent = parent.parentNode; 

   if (parent.nextSibling) { 
      grandparent.insertBefore(pre, parent.nextSibling); 
   } else { grandparent.appendChild(pre); }

   mySelection.collapse(pre.firstChild, 0); 
   mySave(false); // don't prompt, autosave
}

function myPreformatted() {
   var parent = myGetParent(myRange.startContainer); // pre, li, p, h1, h2, h3
   var name = parent.tagName.toLowerCase(); 

   if (myRange.startOffset == myRange.startContainer.length) {
      if (name == 'p' || name == 'li' || 
          name == 'h1' || name == 'h2' || name == 'h3') {
         myNewPreformatted();
      }
   }
}

function myNewHeading(level) { 
   // We only allow the first three levels of headers
   if (!(level == 1 || level == 2 || level == 3)) { return false; }

   var h = document.createElement('h' + level); 
   h.innerHTML = '&nbsp;'; 

   var parent = myGetMainParent(myRange.startContainer); // p, pre, ul, h[1-3]
   var grandparent = parent.parentNode; 

   if (parent.nextSibling) { 
      grandparent.insertBefore(h, parent.nextSibling); 
   } else { grandparent.appendChild(h); }

   mySelection.collapse(h.firstChild, 0); 
   mySave(false); // don't prompt, autosave
}

function myHeading(level) { 
   // We only allow the first three levels of headers
   if (!(level == 1 || level == 2 || level == 3)) { return false; }

   var parent = myGetParent(myRange.startContainer); // pre, li, p, h1, h2, h3
   var name = parent.tagName.toLowerCase();
   if (name == 'pre') {
      myNewHeading(level);
   } else if (myRange.startOffset == myRange.startContainer.length) {
      if (name == 'p' || name == 'li' || 
          name == 'h1' || name == 'h2' || name == 'h3') { 
         myNewHeading(level);
      }
   }
}

function myExitElement(current) { 
   var grandparent = current.parentNode;
   for (var i=0; i<grandparent.childNodes.length; i++) {
      var child = grandparent.childNodes.item(i);
      if (child == current) { 
         var next = grandparent.childNodes.item(i+1);
         mySelectNode(next);
      }
   }
}

function myRemoveInline() { 
   var inline = myFindParent(['em', 'strong', 'a']); 
   if (inline) { 
      var grandparent = inline.parentNode; 

      // Clone the contents of the inline element, and insert it to before 
      // where the current inline element is.
      for (var i=0; i<inline.childNodes.length;i++) { 
         var copy = inline.childNodes.item(i).cloneNode(true); 
         grandparent.insertBefore(copy, inline); 
      }

      // Now remove the current inline element
      grandparent.removeChild(inline); 

      // Select the last of the new clones (hopefully)
      mySelection.collapse(copy, copy.length); 
      myUpdateStatus(); 
   }
}

function myStripWhitespace(s) { 
   s = s.replace(/^[\t\r\n ]*$/, ''); 
   return s.replace(/[\t\n\r ]{2,}$/, ' '); 
}

function myMoveLeft() { 
   data = mySelection.anchorNode.data; 
   stripped = myStripWhitespace(data.substring(0, mySelection.anchorOffset)); 
   mySelection.collapse(mySelection.anchorNode, stripped.length); 
   mySelection.collapse(mySelection.anchorNode, mySelection.anchorOffset - 1); 
   return true; 
}

function myDelete() { 
   // @@ Deleting a paragraph

   mySelection = window.getSelection();
   try { myRange = mySelection.getRangeAt(0); }
   catch(e) { return false; }

   mySelection.removeAllRanges(); 

   var data = myRange.endContainer.data; 
   if (myRange.toString().length == 0 && typeof(data) != 'undefined') { 
      var left = data.substring(0, myRange.endOffset); 
      var right = data.substring(myRange.endOffset, data.length); 
      if (/[^\t\r\n ]/.test(left)) { 
         var stripped = right.replace(/^[\t\r\n ]+/, ' '); 
      } else { stripped = right.replace(/^[\t\r\n ]+$/, ''); }
      myRange.setEnd(myRange.endContainer, 
         myRange.endOffset + (right.length - stripped.length) + 1); 
   }

   myRange.extractContents(); 
   mySelection.removeAllRanges(); 
   mySelection.addRange(myRange); 

   myDeleteHack(); 
}

function myDeleteHack() { 
   // Now time for more &nbsp; hacking!
   // 
   // This arises because if a paragraph contains "pqr s" and the end s is
   // deleted, then the trailing space won't show, which is confusing because
   // then the next time backspace is hit, nothing visibly happens even though
   // that space is deleted.

   mySelection = window.getSelection(); 
   try { myRange = mySelection.getRangeAt(0); }
   catch(e) { return false; }

   // If we have deleted in p, li, h[1-3] and we end with a space, nbspise it
   var current = myFindParent(['p', 'li', 'h1', 'h2', 'h3']); 
   if (current && myRange.startOffset == myRange.startContainer.length) { 
      var html = current.innerHTML; 
      if (html.substr(html.length - 1) == ' ' && html.length > 1) { 
         current.innerHTML = html.substr(0, html.length - 1) + '&nbsp;'; 
         mySelection.collapse(current.lastChild, current.lastChild.length); 
         // myRange = mySelection.getRangeAt(0); 
      }
   }
}

function myPossiblyRemoveParagraph() { 
   var parent = myGetParent(myRange.startContainer); 
   var name = parent.tagName.toLowerCase(); 
   if (name == 'p') { 
      // If the paragraph is now empty, we'll fill it with a non-breaking 
      // space so that it continues to be displayed.
      if (parent.innerHTML == '') { 
         parent.innerHTML = '&nbsp;'; 
         mySelection.collapse(parent.firstChild, 0); 
      }

      // If the paragraph was already "empty", remove it altogether!
      else if (parent.innerHTML == '&nbsp;') { 
         // @@ Will cause an error if deleting to an nbsp
         var grandparent = parent.parentNode; 

         for (var i=0; i<grandparent.childNodes.length; i++) { 
            if (grandparent.childNodes.item(i) == parent) { 
               var prev = grandparent.childNodes.item(i-1); 
               // @@ Will break if the paragraph is the first child

               grandparent.removeChild(parent); 
               // @@ might it not have a last child?
               mySelection.collapse(prev.lastChild, prev.lastChild.length); 
               break; 
            }
         }
      }
   }
}

function myGetLocalPath(originalPath) {
   // This is from TiddlyWiki

   // Remove any location or query part of the URL
   var argPos = originalPath.indexOf("?");
   if(argPos != -1)
           originalPath = originalPath.substr(0,argPos);
   var hashPos = originalPath.indexOf("#");
   if(hashPos != -1)
           originalPath = originalPath.substr(0,hashPos);
   // Convert file://localhost/ to file:///
   if(originalPath.indexOf("file://localhost/") == 0)
           originalPath = "file://" + originalPath.substr(16);

   var localPath;
   if(originalPath.charAt(9) == ":") // pc local file
           localPath = unescape(originalPath.substr(8)).replace(new
        RegExp("/","g"),"\\");
   else if(originalPath.indexOf("file://///") == 0) // FireFox pc network file
                localPath = "\\\\" +
   unescape(originalPath.substr(10)).replace(new RegExp("/","g"),"\\");
   else if(originalPath.indexOf("file:///") == 0) // mac/unix local file
           localPath = unescape(originalPath.substr(7));
   else if(originalPath.indexOf("file:/") == 0) // mac/unix local file
           localPath = unescape(originalPath.substr(5));
   else // pc network file
           localPath = "\\\\" +
        unescape(originalPath.substr(7)).replace(new RegExp("/","g"),"\\");
   return localPath; 
}

function mySave(prompt) { 
   // Mostly taken from TiddlyWiki

   var path = document.location.toString(); 
   if (path.substr(0, 5) != 'file:') { 
      alert('Unable to save this file: not local.'); 
      return false; 
   }

   var local = myGetLocalPath(path); 
   if (true) { // window.Components) { 
      try { 
         var privman = netscape.security.PrivilegeManager; 
         privman.enablePrivilege("UniversalXPConnect"); 

         var f = Components.classes["@mozilla.org/file/local;1"]; 
         var file = f.createInstance(Components.interfaces.nsILocalFile); 
         file.initWithPath(local); 
         if (!file.exists()) { 
            file.create(0, 0644); // @@
         }

         var xml = new XMLSerializer(); 
         // var content = xml.serializeToString(document.documentElement); 
         var content = 
            '<html xmlns="http://www.w3.org/1999/xhtml">\n' + 
            document.documentElement.innerHTML + '\n' + 
            '</html>'; 

         var oname = "@mozilla.org/network/file-output-stream;1"; 
         var o = Components.classes[oname]; 
         var out = o.createInstance(Components.interfaces.nsIFileOutputStream);
         out.init(file, 0x20 | 0x02, 00004, null); 
         out.write(content, content.length); 
         out.flush(); 
         out.close(); 

         if (prompt) { 
            msg = 'Successfully saved ' + content.length + ' bytes to: \n'; 
            alert(msg + local); 
         } else { myAutosaved = true; }

         return true; 
      } catch(e) { alert(e); return false; }
   }
   return false; 
}

function myMakeElement(tag) { 
   var container = myRange.startContainer; 
   var offset = myRange.startOffset; 

   parent = myGetParent(container); 
   if (parent.tagName.toLowerCase() == tag) { 
      myExitElement(parent); 
      return false; 
   }

   var content = document.createTextNode(myRange.toString()); 
   if (myRange.toString().length == 0) { 
      // alert('Nothing was selected'); 
      container.splitText(offset);
      var parent = container.parentNode;

      for (var i=0; i<parent.childNodes.length; i++) {
         if (parent.childNodes.item(i) == container) {
            var next = parent.childNodes.item(i+1);
            var node = document.createElement(tag);
            node.innerHTML = '&nbsp;'; 
            parent.insertBefore(node, next);
            mySelectNode(node.childNodes.item(0)); 
            myDelete(); // @@ hmm!
            break; 
         }
      }

      return node; 
   }

   try { myRange.deleteContents(); }
   catch(e) {}

   if (container.nodeType == container.TEXT_NODE && 
       content.nodeType == content.TEXT_NODE) { 
      container.splitText(offset); 
      var parent = container.parentNode; 

      for (var i=0; i<parent.childNodes.length; i++) { 
         if (parent.childNodes.item(i) == container) { 
            var next = parent.childNodes.item(i+1); 
            var node = document.createElement(tag); 
            node.appendChild(content); 
            parent.insertBefore(node, next); 
         }
      }
   }
   mySelection.addRange(myRange); 
   return node; 
}

function myEmphasis() { 
   myMakeElement('em'); 
}

function myStrong() { 
   myMakeElement('strong'); 
}

function myAnchor() { 
   var uri = prompt('URI', 'http://'); 
   var a = myMakeElement('a'); 
   a.setAttribute('href', uri); 
}

function myTogglePreformatting() { 
   try { var parent = myGetParentBlock(myRange.startContainer); }
   catch(e) { return false; }

   var name = parent.tagName.toLowerCase(); 
   if (name == 'p') { 
      myTransformElement(parent, 'pre'); 
   } else { myTransformElement(parent, 'p'); }
}

function myTransformElement(parent, tag) { 
   var grandparent = parent.parentNode; 
   var node = document.createElement(tag); 

   for (var i=0; i<parent.childNodes.length; i++) { 
      node.appendChild(parent.childNodes.item(i).cloneNode(true)); 
   }

   for (var i=0; i<grandparent.childNodes.length; i++) { 
      if (grandparent.childNodes.item(i) == parent) { 
         grandparent.replaceChild(node, grandparent.childNodes.item(i)); 
      }
   }

   if (node.innerHTML == '&nbsp;') { 
      // myMoveLeft(); 
      // myDelete();
      node.innerHTML = '\n';  
      mySelection.collapse(node.firstChild, 0);	
   } else { mySelection.collapse(node.lastChild, node.lastChild.length); }
   myUpdateStatus(); 
}

function myBindEvents() { 
   document.addEventListener('keypress', myKeypress, false); 
   document.addEventListener('keyup', myKeyup, false); // @@ myUpdateStatus...
   document.addEventListener('click', myUpdateStatus, false); 

   myCaretMode = myGetCaret(); 
   myUpdateStatus(); 
}

window.onload = myBindEvents; 

// [EOF]
